@@ -18,7 +18,9 @@ L{Artist}, L{Release}, and L{Track}.
 
 @author: Matthias Friedrich <matt@mafr.de>
 """
-__revision__ = '$Id: model.py 210 2006-04-03 20:55:20Z matt $'
+from sets import Set
+
+__revision__ = '$Id: model.py 8728 2006-12-17 23:42:30Z luks $'
 
 __all__ = [
 	'VARIOUS_ARTISTS_ID', 'NS_MMD_1', 'NS_REL_1', 'NS_EXT_1', 
@@ -84,7 +86,8 @@ class Entity(object):
 
 	id = property(getId, setId, doc='The MusicBrainz ID.')
 
-	def getRelations(self, targetType=None, relationType=None):
+	def getRelations(self, targetType=None, relationType=None,
+			requiredAttributes=(), direction=None):
 		"""Returns a list of relations.
 
 		If C{targetType} is given, only relations of that target
@@ -101,10 +104,18 @@ class Entity(object):
 
 		You may use the C{relationType} parameter to further restrict
 		the selection. If it is set, only relations with the given
-		relation type are returned.
+		relation type are returned. The C{requiredAttributes} sequence
+		lists attributes that have to be part of all returned relations.
+
+		If C{direction} is set, only relations with the given reading
+		direction are returned. You can use the L{Relation.DIR_FORWARD},
+		L{Relation.DIR_BACKWARD}, and L{Relation.DIR_BOTH} constants
+		for this.
 
 		@param targetType: a string containing an absolute URI, or None
 		@param relationType: a string containing an absolute URI, or None
+		@param requiredAttributes: a sequence containing absolute URIs
+		@param direction: one of L{Relation}'s direction constants
 		@return: a list of L{Relation} objects
 
 		@see: L{Entity}
@@ -117,10 +128,61 @@ class Entity(object):
 				for rel in relList:
 					allRels.append(rel)
 
+		# Filter for direction.
+		#
+		if direction is not None:
+			allRels = [r for r in allRels if r.getDirection() == direction]
+
+		# Filter for relation type.
+		#
 		if relationType is None:
 			return allRels
 		else:
-			return [r for r in allRels if r.getType() == relationType]
+			allRels = [r for r in allRels if r.getType() == relationType]
+
+		# Now filer for attribute type.
+		#
+		tmp = []
+		required = Set(iter(requiredAttributes))
+
+		for r in allRels:
+			attrs = Set(iter(r.getAttributes()))
+			if required.issubset(attrs):
+				tmp.append(r)
+		return tmp
+
+
+	def getRelationTargets(self, targetType=None, relationType=None,
+			requiredAttributes=(), direction=None):
+		"""Returns a list of relation targets.
+
+		The arguments work exactly like in L{getRelations}, but
+		instead of L{Relation} objects, the matching relation
+		targets are returned. This can be L{Artist}, L{Release},
+		or L{Track} objects, depending on the relations.
+
+		As a special case, URL strings are returned if the target
+		is an URL.
+
+		@param targetType: a string containing an absolute URI, or None
+		@param relationType: a string containing an absolute URI, or None
+		@param requiredAttributes: a sequence containing absolute URIs
+		@param direction: one of L{Relation}'s direction constants
+		@return: a list of objects, depending on the relation
+
+		@see: L{getRelations}
+		"""
+		ret = [ ]
+		rels =  self.getRelations(targetType, relationType,
+			requiredAttributes, direction)
+
+		for r in rels:
+			if r.getTargetType() == Relation.TO_URL:
+				ret.append(r.getTargetId())
+			else:
+				ret.append(r.getTarget())
+
+		return ret
 
 
 	def addRelation(self, relation):
@@ -187,6 +249,8 @@ class Artist(Entity):
 		self._endDate = None
 		self._aliases = [ ]
 		self._releases = [ ]
+		self._releasesCount = None
+		self._releasesOffset = None
 
 	def getType(self):
 		"""Returns the artist's type.
@@ -379,6 +443,57 @@ class Artist(Entity):
 		"""
 		self._release.append(release)
 
+	def getReleasesOffset(self):
+		"""Returns the offset of the release list.
+
+		This is used if the release list is incomplete (ie. the web
+		service only returned part of the release for this artist).
+		Note that the offset value is zero-based, which means release
+		C{0} is the first release.
+
+		@return: an integer containing the offset, or None
+
+		@see: L{getReleases}, L{getReleasesCount}
+		"""
+		return self._releasesOffset
+
+	def setReleasesOffset(self, offset):
+		"""Sets the offset of the release list.
+
+		@param offset: an integer containing the offset, or None
+
+		@see: L{getReleasesOffset}
+		"""
+		self._releasesOffset = offset
+
+	releasesOffset = property(getReleasesOffset, setReleasesOffset,
+		doc='The offset of the release list.')
+
+	def getReleasesCount(self):
+		"""Returns the number of existing releases.
+	
+		This may or may not match with the number of elements that
+		L{getReleases} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setReleasesCount}, L{getReleasesOffset}
+		"""
+		return self._releasesCount
+
+	def setReleasesCount(self, value):
+		"""Sets the number of existing releases.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getReleasesCount}, L{setReleasesOffset}
+		"""
+		self._releasesCount = value
+
+	releasesCount = property(getReleasesCount, setReleasesCount,
+		doc='The total number of releases')
+
 
 class Release(Entity):
 	"""Represents a Release.
@@ -409,6 +524,7 @@ class Release(Entity):
 	TYPE_OFFICIAL = NS_MMD_1 + 'Official'
 	TYPE_PROMOTION = NS_MMD_1 + 'Promotion'
 	TYPE_BOOTLEG = NS_MMD_1 + 'Bootleg'
+	TYPE_PSEUDO_RELEASE = NS_MMD_1 + 'Pseudo-Release'
 
 	def __init__(self, id_=None, title=None):
 		"""Constructor.
@@ -429,6 +545,7 @@ class Release(Entity):
 		#self._discIdsCount = None
 		self._tracks = [ ]
 		self._tracksOffset = None
+		self._tracksCount = None
 
 
 	def getTypes(self):
@@ -590,7 +707,7 @@ class Release(Entity):
 
 		@return: a list containing L{Track} objects
 
-		@see: L{getTracksOffset}
+		@see: L{getTracksOffset}, L{getTracksCount}
 		"""
 		return self._tracks
 
@@ -615,7 +732,7 @@ class Release(Entity):
 
 		@return: an integer containing the offset, or None
 
-		@see: L{getTracks}
+		@see: L{getTracks}, L{getTracksCount}
 		"""
 		return self._tracksOffset
 
@@ -624,13 +741,39 @@ class Release(Entity):
 
 		@param offset: an integer containing the offset, or None
 
-		@see: L{getTracksOffset}
+		@see: L{getTracksOffset}, L{setTracksCount}
 		"""
 		self._tracksOffset = offset
 
 	tracksOffset = property(getTracksOffset, setTracksOffset,
 		doc='The offset of the track list.')
 
+	def getTracksCount(self):
+		"""Returns the number of tracks on this release.
+	
+		This may or may not match with the number of elements that
+		L{getTracks} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setTracksCount}, L{getTracks}, L{getTracksOffset}
+		"""
+		return self._tracksCount
+
+	def setTracksCount(self, value):
+		"""Sets the number of tracks on this release.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getTracksCount}, L{setTracksOffset}
+		"""
+		self._tracksCount = value
+
+	tracksCount = property(getTracksCount, setTracksCount,
+		doc='The total number of releases')
+
+
 	def getReleaseEvents(self):
 		"""Returns the list of release events.
 
