@@ -7,7 +7,7 @@ MusicBrainz webservice.
 There are also DOM helper functions in this module used by the parser which
 probably aren't useful to users.
 """
-__revision__ = '$Id: wsxml.py 201 2006-03-27 14:43:13Z matt $'
+__revision__ = '$Id: wsxml.py 8456 2006-09-03 08:52:14Z matt $'
 
 import re
 import logging
@@ -24,6 +24,7 @@ from musicbrainz2.model import NS_MMD_1, NS_REL_1, NS_EXT_1
 __all__ = [
 	'DefaultFactory', 'Metadata', 'ParseError',
 	'MbXmlParser', 'MbXmlWriter',
+	'ArtistResult', 'ReleaseResult', 'TrackResult',
 ]
 
 
@@ -395,6 +396,11 @@ class MbXmlParser(object):
 		for c in _getChildElements(listNode):
 			resultList.append(creator(c))
 
+	def _getListAttrs(self, listNode):
+		offset = _getIntAttr(listNode, 'offset')
+		count = _getIntAttr(listNode, 'count')
+		return (offset, count)
+
 	def _createArtist(self, artistNode):
 		artist = self._factory.newArtist()
 		artist.setId(_getIdAttr(artistNode, 'id', 'artist'))
@@ -413,6 +419,9 @@ class MbXmlParser(object):
 			elif _matches(node, 'alias-list'):
 				self._addArtistAliases(node, artist)
 			elif _matches(node, 'release-list'):
+				(offset, count) = self._getListAttrs(node)
+				artist.setReleasesOffset(offset)
+				artist.setReleasesCount(count)
 				self._addReleasesToList(node, artist.getReleases())
 			elif _matches(node, 'relation-list'):
 				self._addRelationsToEntity(node, artist)
@@ -443,8 +452,9 @@ class MbXmlParser(object):
 			elif _matches(node, 'disc-list'):
 				self._addDiscs(node, release)
 			elif _matches(node, 'track-list'):
-				offset = _getIntAttr(node, 'offset', 0)
+				(offset, count) = self._getListAttrs(node)
 				release.setTracksOffset(offset)
+				release.setTracksCount(count)
 				self._addTracksToList(node, release.getTracks())
 			elif _matches(node, 'relation-list'):
 				self._addRelationsToEntity(node, release)
